<?php

namespace App\Api\V1\Controllers\CP;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\Validator;

use Dingo\Api\Routing\Helpers;
use JWTAuth;
use DB;

use App\Api\V1\Controllers\ApiController;
  
use App\Model\Room\Room;
use App\Model\Group\GroupInvoice;
use App\Model\Group\GroupRoom;
use App\Model\Room\RoomAvaiable;




class DashboardController extends ApiController
{
    use Helpers;
    function index(){
        $totalRoom = Room::count(); 
        $paymeny = GroupInvoice::where('is_paid', 0)->count();
        $avsilableRoom = RoomAvaiable::where('status', 1)->count();
        return [
            'total_room' => $totalRoom, 
            'n_of_availble_rooms' => $avsilableRoom, 
            'n_of_payment' => $paymeny,
        ]; 
    }
    
    function freeRoom(Request $req){

        $data =Room::select('id','type_id','code','deposit','price','zone_id')
        ->with([
            'avaiables:id,room_id,created_at,from,is_booked,status',
            'type:id,name',
            'zone:id,name'
        ]);
       
            if($req->key && $req->key != ''){
                $data = $data->where('code', 'like', '%'.$req->key. '%');
            }
            if($req->price && $req->price != ''){
                $data = $data->where('price', 'LIKE', '%'.$req->price.'%'); 
            }
            $data = $data->whereHas('avaiables', function($q){
                $q->where('status',1);
            });
            $data= $data->orderBy('id', 'desc')->limit(10)->get();   
            return response()->json($data, 200); 
    }
    function pendding(Request $req){
        $data       = GroupInvoice::select('id','group_id','due_date','created_at','due_date','creator_id')
        ->with([
            'details:id,invoice_id,riel_amount,usd_amount,type_id,description', 
            'details.type:id,name',
            'group:id',
            'group.renters:renter_id,group_id,phone,phone_2,name',
            'group.gRooms:group_id,room_id,type_id',
            'group.gRooms.room:id,code,price',
            'user:id,name'
        ])
        ->withCount([
            'details as total_in_usd' => function($query){
                $query->select(DB::raw("SUM(usd_amount) as total_in_usd")); 
            },
            'details as total_in_riel' => function($query){
                $query->select(DB::raw("SUM(riel_amount) as total_in_riel")); 
            }
        ]);
        // ==============================>> Keyword
        if($req->key && $req->key != ''){
            $data = $data->where('id', 'LIKE', '%'.$req->key.'%')
            ->orWhereHas('group', function ($query) use ($req){
                $query->whereHas('renters', function($query) use($req){
                    $query->where('phone', 'LIKE','%'.$req->key.'%');
                });
            })
            ->orWhereHas('group', function ($query) use ($req){
                $query->whereHas('renters', function($query) use($req){
                    $query->where('name', 'LIKE','%'.$req->key.'%');
                });
            })
            ->orWhereHas('group', function ($query) use ($req){
                $query->whereHas('gRooms', function($query) use($req){
                    $query->whereHas('room',function($query) use($req){
                        $query->where('code', 'LIKE','%'.$req->key.'%');
                    });
                });
            }); 
        }  
        $data = $data->where('is_paid',0);
        $data= $data->orderBy('id', 'desc')->limit(8)->get();         
        return response()->json($data, 200); 
    }

   function action(Request $req){

    // ==================================> Book
       $book =GroupRoom::select('id','group_id','room_id','booked_date','deposit','type_id','created_at')
       ->with([
        'room:id,code',
        'type:id,name',
        'group:id',
        'group.gRenters',
        'group.gRenters.renters'
        ]) 
       ->where('type_id',1);
         // ==============================>> Date Range
         if($req->from && $req->to && isValidDate($req->from) && isValidDate($req->to)){
            $book = $book->whereBetween('booked_date', [$req->from." 00:00:00", $req->to." 23:59:59"]);
        }
       $book= $book->orderBy('updated_at', 'desc')->limit(20)->get();  

    // ==================================> Check In
       $checkin = GroupRoom::select('id','group_id','room_id','check_in_date','type_id','created_at')
       ->with([
        'room:id,code',
        'type:id,name',
        'group:id',
        'group.gRenters:id,group_id,renter_id,is_primary_contact',
        'group.gRenters.renters:id,name,phone'
        ]) 
        ->where('type_id',2);
        // ==============================>> Date Range
        if($req->from && $req->to && isValidDate($req->from) && isValidDate($req->to)){
            $checkin = $checkin->whereBetween('check_in_date', [$req->from." 00:00:00", $req->to." 23:59:59"]);
        }
       $checkin= $checkin->orderBy('id', 'desc')->limit(20)->get(); 
     // ==================================> CheCk OUt
       $checkout = GroupRoom::select('id','group_id','room_id','check_out_date','type_id')
       ->with([
        'room:id,code',
        'type:id,name',
        'group:id',
        'group.gRenters:id,group_id,renter_id,is_primary_contact',
        'group.gRenters.renters:id,name,phone'
        ]) 
        ->where('type_id',3,4);
        if($req->from && $req->to && isValidDate($req->from) && isValidDate($req->to)){
            $checkout = $checkout->whereBetween('check_out_date', [$req->from." 00:00:00", $req->to." 23:59:59"]);
        }
       $checkout= $checkout->orderBy('id', 'desc')->limit(20)->get(); 
     // ==================================> Room free
       $room =Room::select('id','type_id','code','deposit','price','zone_id')
       ->with([
           'avaiables:id,room_id,created_at,from,is_booked,status',
           'type:id,name',
           'zone:id,name'
       ]);
      
           if($req->key && $req->key != ''){
               $room = $room->where('code', 'like', '%'.$req->key. '%');
           }
           if($req->price && $req->price != ''){
               $room = $room->where('price', 'LIKE', '%'.$req->price.'%'); 
           }
           $room = $room->whereHas('avaiables', function($q){
               $q->where('status',1);
           });
           $room= $room->orderBy('id', 'desc')->limit(10)->get();  
       return [
        'book' => $book, 
        'check_in' => $checkin, 
        'check_out' => $checkout,
        'room'  => $room,
    ]; 
   }
}