<?php

namespace App\Api\V1\Controllers\CP;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\Validator;
use Illuminate\Validation\Rule;

use Dingo\Api\Routing\Helpers;
use JWTAuth;

use App\Api\V1\Controllers\ApiController;
  
use App\Model\Expense\Expense;
use App\Model\Expense\ExpenseType;

use Carbon\Carbon;

class ExpenseController extends ApiController
{
    use Helpers;
   
    function listing(Request $req){
       
        $data           = Expense::select('*')
        ->with([
            'type:id,name',
            'rate:id,riel'
        ]);
        

         // ==============================>> Type
         if($req->type != 0){
            $data = $data->where('type_id',$req->type);
        }
        // ==============================>> Keyword
        if($req->key && $req->key != ''){
            $data = $data->where('id', 'LIKE', $req->key)->orWhere('referal_id', 'like', $req->key); 
        }
        
        // ==============================>> Date Range
        if($req->from && $req->to && isValidDate($req->from) && isValidDate($req->to)){
            $data = $data->whereBetween('created_at', [$req->from." 00:00:00", $req->to." 23:59:59"]);
        }
        
        $data= $data->orderBy('id', 'desc')->paginate( $req->limit ? $req->limit : 10);         
        return response()->json($data, 200);  
    }

    function view($id=0){
        $data           =Expense::select('*')->find($id);

        if($data){
            return response()->json($data, 200);
        }else{
            return response()->json([
                'status'    =>'fail',
                'message'   =>'Record not found.'
            ], 404);
        }
    }

    function create(Request $req){

        $this->validate($req, [
            
            'type_id'               => 'required',
            'amount'                => 'required',
            'referal_id'            => 'required',  
           
            
        ],[
            'type_id.required'      => 'Please add Expense type',
            'amount.required'       => 'Please add price ',
            'referal_id.required'   => 'Please add referal',

        ]);

        $data                       =   New Expense; 
        $data->type_id              = $req->input('type_id');
        $data->rate_id              = $req->input('rate_id');
        $data->amount               = $req->input('amount');
        $data->referal_id           = $req->input('referal_id');
        $data->description          = $req->input('description');

        $data->save(); 
        
        return response()->json([
            'status' => 'success',
            'menus' => $data,
            'message' => 'ទិន្នន័យដែលបានបង្កើតថ្មី'
        ], 200);
        
    }

    function update(Request $req,  $id = 0 ){
        $this->validate($req, [
            
            'type_id'               => 'required',
            'amount'                => 'required',
            'referal_id'            => 'required',  
        
        ],[
            'type_id.required'      => 'Please add Expense type',
            'amount.required'       => 'Please add price ',
            'referal_id.required'   => 'Please add referal',

        ]);
        $data               = Expense::find($id);
    
        if($data){

            $data->type_id              = $req->input('type_id');
            $data->amount               = $req->input('amount');
            $data->referal_id           = $req->input('referal_id');
            $data->description          = $req->input('description');

            $data->save();

            return response()->json([
                'status'            => 'success',
                'message'           => 'បានបង្កើតដោយជោគជ័យ',
                'Income'            => $data,

            ], 200);
        }else{
            return response()->json([
                'status' => 'No data record',
                'message' => 'Invalid Expense'
            ]);
        }

    }

    function delete($id = 0){
        $menu = Expense::find($id);
        if($menu){

        $menu->delete();

        return response()->json([
            'status' => 'success',
            'message' => 'Data has been deleted',
        ], 200);
        }else{
            return response()->json([
                'status' => 'No data record',
                'message' => 'Invalid Expense'
            ]);
        }
       
    }
    function record(Request $req){
       
        $data           = Expense::select('*')
        ->with([
            'type:id,name',
            'rate:id,riel'
        ]);
         // ==============================>> Date Range
         if($req->from && $req->to && isValidDate($req->from) && isValidDate($req->to)){
            $data = $data->whereBetween('created_at', [$req->from." 00:00:00", $req->to." 23:59:59"]);
        }
        $data= $data->orderBy('id', 'desc')->get();         
        $sum = $data->sum('amount');
        $payload = [
            
            'from'          => $req->from,
            'to'            => $req->to,
            'total_amount'  => $sum,
            'data'          => $data,

        ];
        return $payload;
    }
}