<?php

namespace App\Api\V1\Controllers\CP;

use Illuminate\Http\Request;
use DB;
use Dingo\Api\Routing\Helpers;
use JWTAuth;

use App\Api\V1\Controllers\ApiController;
  
use App\Model\Staff\StaffSalary;
use App\Model\Staff\StaffSalaryDetail;
use App\Model\Staff\StaffSalaryType;
use App\Model\Expense\Expense;

use Carbon\Carbon;

class ExpensePendingController extends ApiController
{
    use Helpers;
   
    function listing(Request $req){
        $data           = StaffSalary::select('*')
        ->with([
            'staff:id,name,phone',
            'details:id,amount,salary_id,type_id',
            'details.type:id,name' 
        ]);
        
      // ==============================>> Key
      if($req->key && $req->key != ''){
        $data = $data->where('id', 'LIKE', '%'.$req->key.'%')
        ->orWhereHas('staff', function ($query) use ($req){
            $query->where('name', 'LIKE','%'.$req->key.'%');
        })
        ->orWhereHas('staff', function ($query) use ($req){
            $query->where('phone', 'LIKE','%'.$req->key.'%');
        }); 
    }  
        // ==============================>> Date Range
        if($req->from && $req->to && isValidDate($req->from) && isValidDate($req->to)){
            $data = $data->whereBetween('created_at', [$req->from." 00:00:00", $req->to." 23:59:59"]);
        }
        $data= $data->where('is_paid',0);
        $data= $data->orderBy('id', 'desc')->paginate( $req->limit ? $req->limit : 10);         
        return response()->json($data, 200);  
    }

    function listtype(){
        $data           =StaffSalaryType::select('id','name')->get();
        return response()->json($data, 200);  
    }

    function view($id=0){
        $data           =StaffSalary::select('*')->find($id);

        if($data){
            return response()->json($data, 200);
        }else{
            return response()->json([
                'status'    =>'fail',
                'message'   =>'Record not found.'
            ], 404);
        }
    }

    function create(Request $req){

        $user = JWTAuth::parseToken()->authenticate();
        $this->validate($req, [
            
            'staff_id'              => 'required',
            'type_id'               => 'required',
            'amount'                => 'required',
        ],[
            'staff_id.required'     => 'Please input staff',
            'type_id.required'      => 'Please input type',
            'amount.required'       => 'Please input amount',
        ]);

        $salary                       =   New StaffSalary; 
        $salary->staff_id             = $req->input('staff_id');
        $salary->rate_id              = $req->input('rate_id');
        $salary->is_paid              = 0;  //Unpaid
        $salary->payer_id             = $user->id;
        $salary->note                 = $req->input('note');

        $salary->save(); 

        $detail                       = New StaffSalaryDetail;
        $detail->salary_id            =$salary->id; 
        $detail->type_id              = $req->input('type_id');
        $detail->amount               = $req->input('amount');
        $detail->description          = $req->input('description');
        $detail->save(); 
        return response()->json([
            'status' => 'success',
            'salary' => $salary,
            'detail' => $detail,
            'message' => 'ទិន្នន័យដែលបានបង្កើតថ្មី'
        ], 200);
        
    }

    function update(Request $req,  $id = 0 ){
        // return 'hi';
        $user = JWTAuth::parseToken()->authenticate();
        $this->validate($req, [
            
            'staff_id'              => 'required',
            'type_id'                  => 'required',
            'amount'                => 'required',
        ],[
            'staff_id.required'     => 'Please input staff',
            'type_id.required'         => 'Please input type',
            'amount.required'       => 'Please input amount',

        ]);
        $data               = StaffSalary::find($id);
    
        if($data){

            // $salary                       =   New StaffSalary; 
            $data->staff_id             = $req->input('staff_id');
            $data->rate_id              = $req->input('rate_id');
            $data->is_paid              = $req->input('is_paid');
            $data->payer_id             = $user->id;
            $data->note                 = $req->input('note');
    
            $data->save(); 
            // return $data;
            $detail = StaffSalaryDetail::where('salary_id', $data->id)->first();
            // return $detail; 
            if($detail){
                $detail                       = StaffSalaryDetail::find($detail->id);
                $detail->salary_id            = $data->id; 
                $detail->type_id              = $req->input('type_id');
                $detail->amount               = $req->input('amount');
                $detail->description          = $req->input('description');
                $detail->save(); 

                return response()->json([
                    'status'            => 'success',
                    'message'           => 'បានបង្កើតដោយជោគជ័យ',
                    'data'              => $data,
                    'detail'            => $detail,
    
                ], 200);
            }else{
                return response()->json([
                    'status' => 'No data record',
                    'message' => 'Invalid detail'
                ]); 
            }
        }else{
            return response()->json([
                'status' => 'No data record',
                'message' => 'Invalid data'
            ]);
        }

    }

    function delete($id = 0){
        $menu = StaffSalary::find($id);
        if($menu){

        $menu->delete();

        return response()->json([
            'status' => 'success',
            'message' => 'Data has been deleted',
        ], 200);
        }else{
            return response()->json([
                'status' => 'No data record',
                'message' => 'Invalid data'
            ]);
        }
       
    }
    function payment( $id = 0){
        $data = StaffSalary::find($id);


        if($data){

        $data->is_paid=1;
        $data->save();
        if($data->is_paid==1){
            $data                      =   New Expense; 
            $data->type_id              = 1;
            $data->referal_id           ="SPM-".$id;
            $data->description          = '';
            
            $data->save(); 

        }
        
        return response()->json([
            'status' => 'success',
            'message' => 'Invoice has been pay ',
        ], 200);
        }else{
            return response()->json([
                'status' => 'No data record',
                'message' => 'Data not found'
            ]);
        }
       
    }
    function record(Request $req){
       
        $data           = StaffSalary::select('id','staff_id','is_paid','payer_id','created_at')
        ->with([
            'staff:id,name,phone',
            'details:id,amount,salary_id,type_id',
            'details.type:id,name' 
        ])
        ->withCount([
            'details as total' => function($query){
                $query->select(DB::raw("SUM(amount) as total")); 
            }
        ]);
         // ==============================>> Date Range
         if($req->from && $req->to && isValidDate($req->from) && isValidDate($req->to)){
            $data = $data->whereBetween('created_at', [$req->from." 00:00:00", $req->to." 23:59:59"]);
        }
        $data= $data->where('is_paid',0);
        $data= $data->orderBy('id', 'desc')->get();         
        $sum = $data->sum('total');
        $payload = [
            
            'from'          => $req->from,
            'to'            => $req->to,
            'total_amount'  => $sum,
            'data'          => $data,

        ];
        return $payload;
    }
}