<?php

namespace App\Api\V1\Controllers\CP;

use Illuminate\Http\Request;

use Dingo\Api\Routing\Helpers;
use DB;


use App\Api\V1\Controllers\ApiController;
  
use App\Model\Group\Group;
use App\Model\Group\GroupRenter;
use App\Model\Renter\Renter;
use App\Model\Income\Income;
use App\Model\Group\GroupInvoice;
use App\Model\Group\GroupRoom;
use Carbon\Carbon;

class GroupRenterController extends ApiController
{
    use Helpers;
   
    function listing(Request $req){
       
        $data           = Group::select('*')
        ->withcount([
            'renters'
        ]);
       
        if( $req->id && $req->id !="" ){
            $data = $data->where('id', 'like','%'.$req->id.'%');
        }

        if($req->from && $req->to && isValidDate($req->from) && isValidDate($req->to)){
            $data = $data->whereBetween('updated_at', [$req->from." 00:00:00", $req->to." 23:59:59"]);
        }

        $data= $data->orderBy('id', 'desc')->paginate( $req->limit ? $req->limit : 10);         
        return response()->json($data, 200);  
    }

    function view($id=0){
        $data            =Group::select('*')
        ->withcount([
            'renters'
        ])
        ->find($id);
        if($data){
            return response()->json($data, 200);
        }else{
            return response()->json([
                'status'    =>'fail',
                'message'   =>'Record not found.'
            ], 404);
        }
    }
    function create(Request $req){

        $this->validate($req, [
            
            'group_id'                  => 'required',
            'renter_id'                 => 'required',
            'is_primary_contact'        => 'required',
            
        ],[
            'group_id.required'             => 'Please add id of group',
            'renter_id.required'            => 'Please add id of renter',
            'is_primary_contact.required'   => 'Please add primary contact',
        ]);

        $data                           =   New GroupRenter; 
        $data->group_id                 = $req->input('group_id');
        $data->renter_id                = $req->input('renter_id');
        $data->is_primary_contact       = $req->input('is_primary_contact');

        $data->save(); 
        
        return response()->json([
            'status' => 'success',
            'Staff' => $data,
            'message' => 'ទិន្នន័យដែលបានបង្កើតថ្មី'
        ], 200);
        
    }
    
    function update(Request $req,  $id = 0 ){
        $this->validate($req, [
            
            'group_id'                  => 'required',
            'renter_id'                 => 'required',
            'is_primary_contact'        => 'required',
           
            
        ],[
            'group_id.required'             => 'Please add id of group',
            'renter_id.required'            => 'Please add id of renter',
            'is_primary_contact.required'   => 'Please add primary contact',
        ]);

        $data               = GroupRenter::find($id);
    
        if($data){

        $data->group_id                     = $req->input('group_id');
        $data->renter_id                    = $req->input('renter_id');
        $data->is_primary_contact           = $req->input('is_primary_contact');
        
        
        $data->save();

            return response()->json([
                'status'            => 'success',
                'message'           => 'បានបង្កើតដោយជោគជ័យ',
                'Room'              => $data,

            ], 200);
        }else{
            return response()->json([
                'status' => 'Group renter not record',
                'message' => 'Group renter not record'
            ]);
        }

    }

    function deleteRenter($id = 0){
        $data = GroupRenter::find($id);
        if($data){

        $data->delete();

        return response()->json([
            'status' => 'success',
            'message' => 'Data has been deleted',
        ], 200);
        }else{
            return response()->json([
                'status' => 'Data not record',
                'message' => 'Invalid Group renter'
            ]);
        }
       
    }
    function delete($id = 0){
        $data = Group::find($id);
        if($data){

        $data->delete();

        return response()->json([
            'status' => 'success',
            'message' => 'Data has been deleted',
        ], 200);
        }else{
            return response()->json([
                'status' => 'Data not record',
                'message' => 'Invalid Group renter'
            ]);
        }
       
    }

    function renter($id=0, Request $req){
        
        $data                       =Group::select('*')
        ->with([

            'gRenters.renters'
        ])
        ->where('id',$id);

        if($data){
        $data= $data->orderBy('id', 'desc')->paginate( $req->limit ? $req->limit : 10);         
         return response()->json($data, 200); 
        
        }else{
            return response()->json([
                'status'    =>'fail',
                'message'   =>'Record not found.'
            ], 404);
        }
    }
    function room($id=0, Request $req){
        
        
        $data                       =GroupRoom::select('id','room_id','group_id','deposit','type_id','check_in_date','check_out_date','created_at')
        ->with([

            'room:id,code',
            'type:id,name',
            'group'
        ])
        ->where('group_id',$id);
    
        if($data){
        $data= $data->orderBy('id', 'desc')->paginate( $req->limit ? $req->limit : 10);         
         return response()->json($data, 200); 
        
        }else{
            return response()->json([
                'status'    =>'fail',
                'message'   =>'Record not found.'
            ], 404);
        }
    }
    function payment($id=0, Request $req){
        
        $data                       =GroupInvoice::select('*')
        ->with([
            'details:id,invoice_id,riel_amount,usd_amount,type_id,description', 
            'details.type:id,name',
            'group:id',
            'group.renters:renter_id,group_id,phone,phone_2,name',
            'group.gRooms:group_id,room_id,type_id',
            'group.gRooms.room:id,code,price',
            'user:id,name'
        ])
        ->withCount([
            'details as total_in_usd' => function($query){
                $query->select(DB::raw("SUM(usd_amount) as total_in_usd")); 
            },
            'details as total_in_riel' => function($query){
                $query->select(DB::raw("SUM(riel_amount) as total_in_riel")); 
            }
        ])
        ->where('group_id',$id);
        if($data){
        $data= $data->orderBy('id', 'desc')->get();         
         return response()->json($data, 200); 
        
        }else{
            return response()->json([
                'status'    =>'fail',
                'message'   =>'Record not found.'
            ], 404);
        }
    }
    function file($id=0, Request $req){
        
        
        $data                       =GroupRenter::select('renter_id')
        ->with([

            'renters:id,photo'
        ]);
    
        if($data){
        $data= $data->orderBy('id', 'desc')->paginate( $req->limit ? $req->limit : 10);         
         return response()->json($data, 200); 
        
        }else{
            return response()->json([
                'status'    =>'fail',
                'message'   =>'Record not found.'
            ], 404);
        }
    }
    function createRenter(Request $req, $id=0){
      $group          = Group::find($id);
       if($group){
            $name           =$req->input('name');
            $phone          =$req->input('phone');
            $phone2         =$req->input('phone2');
            $email          =$req->input('email');
            $address        =$req->input('address'); 
            $check_in_date  =$req->input('check_in_date');
            $renter_id = 0;
            $group_id = 0;

            $renter = Renter::select('*')->where('phone', $phone)->first();
            if($renter){ //Renter has already existed
                //Find that renter
                // $renter_id = $renter->id;

                
                $data                           =   New GroupRenter; 
                $data->group_id                 = $id;
                $data->renter_id                = $renter->id;
                $data->is_primary_contact       = 1;
                $data->check_in_date            = $check_in_date;
        
                $data->save(); 

                return response()->json([
                    'status' => 'success',
                    'message' => 'លេខនេះត្រូវបានប្រើប្រាស់រួចហើយ',
                    'Renter' => $renter,
                    'Group'  => $data
                ], 200);
                
            }
            else{
        
                $data                       = New Renter; 
                $data->name                 = $name;
                $data->phone                = $phone;
                $data->phone_2              = $phone2;
                $data->email                = $email;
                $data->address              = $address;
                $data->save(); 
                $rGroup                     = New GroupRenter;
                $rGroup->group_id           = $id;
                $rGroup->renter_id          = $data->id;
                $rGroup->is_primary_contact = 1;
                $rGroup->check_in_date      = $check_in_date;
                $rGroup->save();
                
                return response()->json([
                    'status' => 'success',
                    'Renter' => $data,
                    'message' => 'ទិន្នន័យដែលបានបង្កើតថ្មី'
                ], 200);
            }
       }

       
    }
    function updateRenter(Request $req, $id=0){
        $this->validate($req, [
            'name'              => 'required|max:50',
            'phone'             =>  [
                                     'required', 
                                     'regex:/(^[0][0-9].{7}$)|(^[0][0-9].{8}$)/', 
                                    ], 
 

                            ],[
            'name.required'         => 'Please input name',
            'phone.required'        => 'Please add Phone number',
            'phone.regex'           => 'Please input correct number',

                            ]);
        $data               = Renter::find($id);
        
        if($data){

            $data->name                 = $req->input('name');
            $data->phone                = $req->input('phone');
            $data->phone_2              = $req->input('phone2');
            $data->email                = $req->input('email');
            $data->address              = $req->input('address');
            $data->save();

            return response()->json([
                'status'            => 'success',
                'message'           => 'បានកែប្រែដោយជោគជ័យ',
                'data'            => $data

            ], 200);
        }else{
            return response()->json([
                'status' => 'Data not record',
                'message' => 'Invalid renter'
            ]);
        }
    }
    function checkout(Request $req, $id=0){
        
        $data               = GroupRenter::find($id);
        
        if($data){

            $data->check_out_date                 = $req->input('check_out_date');
            $data->save();

            return response()->json([
                'status'            => 'success',
                'message'           => 'បានចាក់ចេញ',
                'data'            => $data

            ], 200);
        }else{
            return response()->json([
                'status' => 'Data not record',
                'message' => 'Invalid data'
            ]);
        }
    }
    
}