<?php

namespace App\Api\V1\Controllers\CP;

use Illuminate\Http\Request;
use Dingo\Api\Routing\Helpers;
use App\Api\V1\Controllers\ApiController;
use App\Model\Group\GroupInvoice;
use App\Model\Income\Income;
use DB;
class IncomeController extends ApiController
{
    use Helpers;
   
    function listing(Request $req){
       
        $data           = Income::select('id','type_id','rate_id','created_at','usd_amount','referal_id','riel_amount')
        ->with([
            'type:id,name',
            'rate:id,riel'
        ]);

         // ==============================>> Type
        if($req->type != 0){
            $data = $data->where('type_id',$req->type);
        }

        // ==============================>> Keyword
        if($req->key && $req->key != ''){
            $data = $data->where('id', 'LIKE', $req->key)->orWhere('referal_id', 'like','%'.$req->key.'%'); 
        }
        // ==============================>> user
        if($req->user && $req->user != ''){
            $data = $data->where('creator_id', 'LIKE', $req->user); 
        }
        
        // ==============================>> Date Range
        if($req->from && $req->to && isValidDate($req->from) && isValidDate($req->to)){
            $data = $data->whereBetween('created_at', [$req->from." 00:00:00", $req->to." 23:59:59"]);
        }
        
        $data= $data->orderBy('id', 'desc')->paginate( $req->limit ? $req->limit :10);               
        return response()->json($data, 200); 
    }

    function view($id=0){
        $Income            =Income::select('*')
        ->with([
            'type:id,name',
            'rate:id,riel'
        ])
        ->find($id);

        if($Income){
            return response()->json($Income, 200);
        }else{
            return response()->json([
                'status'    =>'fail',
                'message'   =>'Record not found.'
            ], 404);
        }
    }

    function create(Request $req ){

        $this->validate($req, [
            
            'type_id'               => 'required',
            'usd_amount'            => 'required|numeric',
            'riel_amount'           => 'required|numeric',
            
        ],[
            'type_id.required'      => 'Please add type',
            'usd_amount.required'   => 'Please input amount USD',
            'usd_amount.numeric'   => 'Amount must be number',
            'riel_amount.required'  => 'Please input amount Riel',
            'riel_amount.numeric'  => 'Amount must be number',
        ]);

        $data                      =   New Income; 
        $data->type_id              = $req->input('type_id');
        $data->rate_id              = 1;
        $data->usd_amount           = $req->input('usd_amount');
        $data->riel_amount          = $req->input('riel_amount');
        $data->referal_id           = $req->input('referal_id');
        $data->description          = $req->input('description');
        
        $data->save(); 
        
        return response()->json([
            'status' => 'success',
            'menus' => $data,
            'message' => 'ទិន្នន័យដែលបានបង្កើតថ្មី'
        ], 200);
        
    }

    function update(Request $req,  $id = 0 ){
        $this->validate($req, [
            
            'type_id'               => 'required',
            'rate_id'               => 'required',
            
        ],[
            'type_id.required'      => 'Please add type',
            'rate_id.required'      => 'Please add rate',
        ]);
        $data                           = Income::find($id);
    
        if($data){

            $data->type_id              = $req->input('type_id');
            $data->rate_id              = 1;
            $data->usd_amount           = $req->input('usd_amount');
            $data->riel_amount          = $req->input('riel_amount');
            $data->referal_id           = $req->input('referal_id');
            $data->description          = $req->input('description');

            $data->save();

            return response()->json([
                'status'            => 'success',
                'message'           => 'បានបង្កើតដោយជោគជ័យ',
                'Income'            => $data,

            ], 200);
        }else{
            return response()->json([
                'status' => 'No data record',
                'message' => 'Invalid Income'
            ]);
        }

    }

    function delete($id = 0){
        $data = Income::find($id);
        if($data){

        $data->delete();

        return response()->json([
            'status' => 'success',
            'message' => 'Data has been deleted',
        ], 200);
        }else{
            return response()->json([
                'status' => 'No data record',
                'message' => 'Data not found'
            ]);
        }
       
    }
    function unpaid($id = 0, Request $req){
        $data = Income::find($id);
        //return $data;
        if($data){
            if($data->deleted_at==null){
                $pieces = explode("-", $data->referal_id);
                //return $pieces[1];
                $unpaid               = GroupInvoice::where('id',$pieces[1])->first(); 
                //return $unpaid;
                if($unpaid){
                    if( $unpaid->total_received_usd == $data->usd_amount && $unpaid->total_received_riel ==$data->riel_amount ){
                        
                        $unpaid->is_paid= 0;
                        $unpaid->total_received_usd=$unpaid->total_received_usd+$data->usd_amount;
                        $unpaid->total_received_riel=$unpaid->total_received_riel+$data->riel_amount;
                        $unpaid->debt_usd = null;
                        $unpaid->debt_riel = null;
                        $unpaid->save(); 
    
                        $data->delete();
    
                        return response()->json([
                            'status'  => 'success',
                            'message' => 'invoice have been update to unpaid',
                            'Unpaid'  => $unpaid,
                           
                        ], 200);
                    }else{
                        
                        $unpaid->is_paid= 0;
                        $unpaid->total_received_usd=$unpaid->total_received_usd-$data->usd_amount;
                        $unpaid->total_received_riel=$unpaid->total_received_riel-$data->riel_amount;
                        $unpaid->debt_usd = $unpaid->debt_usd + $data->usd_amount;
                        $unpaid->debt_riel = $unpaid->debt_riel + $data->riel_amount;
                        $unpaid->save(); 

                        $data->delete();

                        return response()->json([
                            'status'  => 'success',
                            'message' => 'invoice have been update to unpaid',
                            'Unpaid'  => $unpaid,
                            'data'    => $data
                        ], 200);
                    }
                }else{
                    return response()->json([
                        'status' => 'No data record',
                        'message' => 'Data not found'
                    ]);
                }
                
            }else{
                return response()->json([
                    'status' => 'No data record',
                    'message' => 'Data not found'
                ]);
            }
            $data->delete(); 
        }else{
            return response()->json([
                'status' => 'No data record',
                'message' => 'Data not found'
            ]);
        }
       
    }
    function record(Request $req){
      
        $data           = Income::select('id','type_id','rate_id','created_at','usd_amount','referal_id','riel_amount')
        ->with([
            'type:id,name',
            'rate:id,riel'
        ]);
        // ==============================>> Date Range
        if($req->from && $req->to && isValidDate($req->from) && isValidDate($req->to)){
            $data = $data->whereBetween('created_at', [$req->from." 00:00:00", $req->to." 23:59:59"]);
        }
        $data= $data->orderBy('id', 'desc')->get();  
        $sumUSD = $data->sum('usd_amount');
        $sumRiel = $data->sum('riel_amount'); 
        $payload = [
            
            'from'          => $req->from,
            'to'            => $req->to,
            'total_usd'     => $sumUSD,
            'total_riel'    => $sumRiel,
            'data'          => $data,

        ];
        return $payload;
    }

}