<?php

namespace App\Api\V1\Controllers\CP;

use Illuminate\Http\Request;
use DB; 

use Dingo\Api\Routing\Helpers;
use JWTAuth;


use App\Api\V1\Controllers\ApiController;
  
use App\Model\Income\Income;
use App\Model\Group\GroupInvoice;
use App\Model\Group\GroupInvoiceDetail;


class IncomeUnpaidController extends ApiController
{
    use Helpers;
   
    function listing(Request $req){
        
        $data       = GroupInvoice::select('id','group_id','due_date','created_at','due_date','creator_id','is_paid','debt_usd','debt_riel','note')
        ->with([
            'details:id,invoice_id,riel_amount,usd_amount,type_id,description', 
            'details.type:id,name',
            'group:id',
            'group.renters:renter_id,group_id,phone,phone_2,name',
            'group.gRooms:group_id,room_id,type_id',
            'group.gRooms.room:id,code,price',
            'user:id,name'
        ])
        ->withCount([
            'details as total_in_usd' => function($query){
                $query->select(DB::raw("SUM(usd_amount) as total_in_usd")); 
            },
            'details as total_in_riel' => function($query){
                $query->select(DB::raw("SUM(riel_amount) as total_in_riel")); 
            }
        ]);
        if($req->key && $req->key !=0){
            // $pieces = explode("-", $req->key);
            $data = $data->where('P-'.'id', 'LIKE', '%'.$req->key.'%'); 
        }  
        if($req->room && $req->room !=''){
            $data = $data->whereHas('group', function ($query) use ($req){
                $query->whereHas('gRooms', function($query) use($req){
                    $query->whereHas('room',function($query) use($req){
                        $query->where('code', 'LIKE','%'.$req->room.'%');
                    });
                });
            });
        }
        // ==============================>> Keyword
        if($req->renter && $req->renter != ''){
            $data = $data->where('id', 'LIKE', '%'.$req->renter.'%')
            ->orWhereHas('group', function ($query) use ($req){
                $query->whereHas('renters', function($query) use($req){
                    $query->where('name', 'LIKE','%'.$req->renter.'%');
                });
            })
            ->orWhereHas('group', function ($query) use ($req){
                $query->whereHas('renters', function($query) use($req){
                    $query->where('phone', 'LIKE','%'.$req->renter.'%');
                });
            });
        }  
        
        // ==============================>> Date Range
        if($req->from && $req->to && isValidDate($req->from) && isValidDate($req->to)){
            $data = $data->whereBetween('created_at', [$req->from." 00:00:00", $req->to." 23:59:59"]);
        }
        
        $data = $data->where('is_paid',0);
        $data= $data->orderBy('id', 'desc')->paginate( $req->limit ? $req->limit : 10);         
        return response()->json($data, 200); 
    }

    function view($id = 0, Request $req){
      
        $data            =GroupInvoiceDetail::select('id','type_id','description','usd_amount','riel_amount','invoice_id')
        ->with([
            'type:id,name'
        ])
       
        ->where('invoice_id',$id);

        if($data){
            
           $data= $data->orderBy('id', 'desc')->get();
            return response()->json($data, 200);

        }else{
            return response()->json([
                'status'    =>'fail',
                'message'   =>'Record not found.'
            ], 404);
        }
    }

    function create(Request $req){

        $this->validate($req, [
            
            'type_id'               => 'required',
            'usd_amount'            => 'required|numeric',
            'riel_amount'           => 'required|numeric',
            
        ],[
            'type_id.required'      => 'Please add type',
            'usd_amount.required'   => 'Please input amount USD',
            'usd_amount.numeric'   => 'Amount must be number',
            'riel_amount.required'  => 'Please input amount Riel',
            'riel_amount.numeric'  => 'Amount must be number',
        ]);

        $data                      =   New GroupInvoice; 
        $data->type_id              = $req->input('type_id');
        $data->rate_id              = 2;
        $data->is_paid              = 0;
        $data->usd_amount           = $req->input('usd_amount');
        $data->riel_amount          = $req->input('riel_amount');
        $data->receiver_id          = $req->input('receiver_id');
        $data->description          = $req->input('description');
        
        $data->save(); 
        
        return response()->json([
            'status' => 'success',
            'menus' => $data,
            'message' => 'ទិន្នន័យដែលបានបង្កើតថ្មី'
        ], 200);
        
    }

    function delete($id = 0){
        $data = GroupInvoice::find($id);
        if($data){

        $data->delete();

        return response()->json([
            'status' => 'success',
            'message' => 'Data has been deleted',
        ], 200);
        }else{
            return response()->json([
                'status' => 'No data record',
                'message' => 'Data not found'
            ]);
        }
       
    }
    function payment(Request $req, $id = 0){
        $user = JWTAuth::parseToken()->authenticate();
        // $pieces = explode("-",$id);
        $data = GroupInvoice::where('id',$id)->where('is_paid',0)->first();
        $debtUsd = $req->input('debtUsd');
        $debtRiel = $req->input('debtRiel');
      //return $data;
        if($data){
 
            $details = GroupInvoice::find($id)->details;
            $AmountUSD = 0;
            $AmountRiel = 0;
            $totalAmountUSD = $debtUsd;
            $totalAmountRiel = $debtRiel;
            
            foreach($details as $detail){
                $AmountUSD += $detail->usd_amount;
                $AmountRiel += $detail->riel_amount;
            }
            // return $AmountRiel;
            if($data->debt_usd ==null && $data->debt_riel ==null){
                
                    if($AmountUSD == $debtUsd && $AmountRiel == $debtRiel){

                        $income                     =   New Income; 
                        $income->type_id            = 1;
                        $income->rate_id            = 2;
                        $income->usd_amount         = $debtUsd;
                        $income->riel_amount        = $debtRiel;
                        $income->referal_id         = 'P-'.$id;
                        $income->description        = '';
                        $income->creator_id         = $user->id;
                        $income->save(); 
        
                        $data->is_paid              = 1;
                        $data->total_received_riel     = $debtRiel;
                        $data->total_received_usd      = $debtUsd;
                        $data->save();
                        return response()->json([
                            'status' => 'success',
                            'message' => 'Invoice has been pay ',
                            'data'  => $data,
                            'income' => $income,
                        ], 200);
                    }else{
                    
                        $usd = $AmountUSD-intval($debtUsd);
                        $riel = $AmountRiel-intval($debtRiel);
        
                        $data->total_received_riel  = $debtRiel;
                        $data->total_received_usd   = $debtUsd;
                        $data->debt_usd             = $usd;
                        $data->debt_riel            = $riel;
                        $data->note                 = 'ជំពាក់នៅថ្ងៃទី  '.now();
                        $data->save();
                        
                        $income                     =   New Income; 
                        $income->type_id            = 1;
                        $income->rate_id            = 2;
                        $income->usd_amount         = $debtUsd;
                        $income->riel_amount        = $debtRiel;
                        $income->referal_id         = 'P-'.$id;
                        $income->description        = '';
                        $income->creator_id         = $user->id;
                        $income->save(); 
        
                    
                        return response()->json([
                            'status' => 'success',
                            'message' => 'Invoice has been pay ',
                            'data'  => $data,
                            'income' => $income,
                        ], 200);
        
                    }

                }else{

                    if($data->debt_usd > $debtUsd || $data->debt_riel > $debtRiel){
                        //return 'bb';
                        $usd = $data->debt_usd-intval($debtUsd);
                        $riel = $data->debt_riel-intval($debtRiel);

                        $income                     =   New Income; 
                        $income->type_id            = 1;
                        $income->rate_id            = 2;
                        $income->usd_amount         = $debtUsd;
                        $income->riel_amount        = $debtRiel;
                        $income->referal_id         = 'P-'.$id;
                        $income->description        = '';
                        $income->creator_id         = $user->id;
                        $income->save(); 

                        
                        $data->total_received_riel     = $data->total_received_riel+$debtRiel;
                        $data->total_received_usd      = $data->total_received_usd+$debtUsd;
                        $data->debt_usd                = $usd;
                        $data->debt_riel               = $riel;
                        $data->save();
                        return response()->json([
                            'status' => 'success',
                            'message' => 'Invoice has been pay ',
                            'data'  => $data,
                            'income' => $income,
                        ], 200);
                    }else{
                        //return 'cc';
                        $income                     =   New Income; 
                        $income->type_id            = 1;
                        $income->rate_id            = 2;
                        $income->usd_amount         = $debtUsd;
                        $income->riel_amount        = $debtRiel;
                        $income->referal_id         = 'P-'.$id;
                        $income->description        = '';
                        $income->creator_id         = $user->id;
                        $income->save(); 

                        $data->is_paid                 = 1;
                        $data->total_received_riel     = $data->total_received_riel+$debtRiel;
                        $data->total_received_usd      = $data->total_received_usd+$debtUsd;
                        $data->debt_usd                = 0;
                        $data->debt_riel               = 0;
                        $data->save();
                        return response()->json([
                            'status' => 'success',
                            'message' => 'Invoice has been pay ',
                            'data'  => $data,
                            'income' => $income,
                        ], 200);
                    }
                        
                }
            
        }else{
            return response()->json([
                'status' => 'No data record',
                'message' => 'Data not found'
            ]);
        }
       
    }
    
    function record(Request $req){
        
        $data       = GroupInvoice::select('id','group_id','due_date','created_at','due_date','creator_id','is_paid')
        ->with([
            'details:id,invoice_id,riel_amount,usd_amount,type_id,description', 
            'details.type:id,name',
            'group:id',
            'group.renters:renter_id,group_id,phone,phone_2,name',
            'group.gRooms:group_id,room_id,type_id',
            'group.gRooms.room:id,code,price',
            'user:id,name'
        ])
        ->withCount([
            'details as total_in_usd' => function($query){
                $query->select(DB::raw("SUM(usd_amount) as total_in_usd")); 
            },
            'details as total_in_riel' => function($query){
                $query->select(DB::raw("SUM(riel_amount) as total_in_riel")); 
            }
        ]);
          // ==============================>> Date Range
          if($req->from && $req->to && isValidDate($req->from) && isValidDate($req->to)){
            $data = $data->whereBetween('created_at', [$req->from." 00:00:00", $req->to." 23:59:59"]);
        }
        $data = $data->where('is_paid',0);
        $data= $data->orderBy('id', 'desc')->get(); 
        $sumUSD = $data->sum('total_in_usd');
        $sumRiel = $data->sum('total_in_riel'); 
        $payload = [
            
            'from'          => $req->from,
            'to'            => $req->to,
            'total_usd'     => $sumUSD,
            'total_riel'    => $sumRiel,
            'data'          => $data,

        ];
        return $payload; 
    }

    function generateCode(){
        $code = 'P-'.rand(1000000, 9999999); 
        $check = GroupInvoice::where('code', $code)->first(); 
        if($check){
            return $this->generateCode();
        }

        return $code; 
    }

}