<?php

namespace App\Api\V1\Controllers\CP;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\Validator;
use Illuminate\Validation\Rule;
use App\CamCyber\FileUpload;
use Carbon\Carbon;
use Dingo\Api\Routing\Helpers;
use JWTAuth;


use App\Api\V1\Controllers\ApiController;
  
use App\Model\Renter\Renter;
use App\Model\Group\GroupRenter;



class RenterController extends ApiController
{
    use Helpers;
   
    function listing(Request $req){
       
        $data            =Renter::select('id','name','phone','phone_2','email','address','created_at','photo')
        ->with([
        'gRenter:id,renter_id,group_id,is_primary_contact'
        ]);
        // ==============================>> Keyword
        if($req->key && $req->key != ''){
            $data = $data->where('name', 'LIKE', '%'.$req->key.'%')->orWhere('email', 'like', $req->key)->orWhere('phone', 'like', $req->key); 
        }

        $data= $data->orderBy('id', 'desc')->paginate( $req->limit ? $req->limit : 10);         
        return response()->json($data, 200); 
    }

    function view($id=0){
        $data            =Renter::select('id','name','phone','phone_2','email','address','created_at','photo')
        ->with([
        'gRenter:id,renter_id,group_id,is_primary_contact'
        ])
        ->find($id);

        if($data){
            return response()->json($data, 200);
        }else{
            return response()->json([
                'status'    =>'fail',
                'message'   =>'Record not found.'
            ], 404);
        }
    }

    function create(Request $req){

        $this->validate($req, [
            
            'name'                  => 'required|max:50',
            'phone'                 =>  [
                                         'required', 
                                         'regex:/(^[0][0-9].{7}$)|(^[0][0-9].{8}$)/', 
                                        ]
        ],[
            'name.required'         => 'Please add name',
            'phone.required'        => 'Please add Phone number',
            'phone.regex'           => 'Please input correct number',
           


        ]);

        $data                       =   New Renter; 
        $data->name                 = $req->input('name');
        $data->phone                = $req->input('phone');
        $data->phone_2              = $req->input('phone2');
        $data->email                = $req->input('email');
        $data->address              = $req->input('address');

        $date_today = Carbon::today()->format('d') . '-' . Carbon::today()->format('M') . '-' . Carbon::today()->format('Y');
        if ($req->photo) {
            $image     = FileUpload::uploadFileV2($req->photo, 'renter/' . $date_today, '');
            if ($image['url']) {
                $data->photo = $image['url'];
            }
        }

        $data->save(); 
        
        return response()->json([
            'status' => 'success',
            'Renter' => $data,
            'message' => 'ទិន្នន័យដែលបានបង្កើតថ្មី'
        ], 200);
        
    }

    function update(Request $req,  $id = 0 ){
        $this->validate($req, [
            'name'              => 'required|max:50',
            'phone'             =>  [
                                     'required', 
                                     'regex:/(^[0][0-9].{7}$)|(^[0][0-9].{8}$)/', 
                                    ], 
 

                            ],[
                                    'name.required'         => 'Please input name',
                                    'phone.required'        => 'Please add Phone number',
                                    'phone.regex'           => 'Please input correct number',
                            ]);
        $data               = Renter::find($id);
    
        if($data){

            $data->name                 = $req->input('name');
            $data->phone                = $req->input('phone');
            $data->phone_2              = $req->input('phone2');
            $data->email                = $req->input('email');
            $data->address              = $req->input('address');;
            $date_today = Carbon::today()->format('d') . '-' . Carbon::today()->format('M') . '-' . Carbon::today()->format('Y');
            if ($req->photo) {
                $image     = FileUpload::uploadFileV2($req->photo, 'renter/' . $date_today, '');
                if ($image['url']) {
                    $data->photo = $image['url'];
                }
            }
 

            $data->save();

            return response()->json([
                'status'            => 'success',
                'message'           => 'បានបង្កើតដោយជោគជ័យ',
                'Income'            => $data,

            ], 200);
        }else{
            return response()->json([
                'status' => 'No Expense record',
                'message' => 'Invalid renter'
            ]);
        }

    }

    function delete($id = 0){
        $menu = Renter::find($id);
        if($menu){

        $menu->delete();

        return response()->json([
            'status' => 'success',
            'message' => 'Menu has been deleted',
        ], 200);
        }else{
            return response()->json([
                'status' => 'No Expense record',
                'message' => 'Invalid renter'
            ]);
        }
       
    }

   function generateCode(){
        
        $lastCode = Renter::select('id')->orderBy('id', 'DESC')->first(); 
        $code = 1;
        if($lastCode){
            $code = $lastCode->id+1; 
        }

        if( $code >= 1 && $code < 10 ){
            $code = '00000'.$code; 
        }else  if( $code >= 10 && $code < 100 ){
             $code = '0000'.$code; 
        }else  if( $code >= 100 && $code < 100000 ){
             $code = '000'.$code; 
        }else  if( $code >= 100000 && $code < 1000000 ){
             $code = '00'.$code; 
        }
        return $code; 
   }
    
   function getGroup($id=0){
    $data            =Renter::select('id','name','phone','phone_2','email','address','created_at')
    ->with([
        'gRenter:id,renter_id,group_id,is_primary_contact,check_in_date,check_out_date'
    ])
    ->find($id);
    

    if($data){
      
        return response()->json($data, 200);
    }else{
        return response()->json([
            'status'    =>'fail',
            'message'   =>'Record not found.'
        ], 404);
    }
   }
//    function 

}