<?php

namespace App\Api\V1\Controllers\CP;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\Validator;
use Illuminate\Validation\Rule;
use App\Api\V1\Controllers\ApiController;
use App\Model\Expense\Expense;
use Dingo\Api\Routing\Helpers;
use JWTAuth;



use Auth;
use Session;
use App\Model\Room\Room;
use App\Model\Room\Rate;
use App\Model\Room\RoomType;
use App\Model\Zone\Zone;
use App\Model\Room\RoomAvaiable;
use App\Model\Group\GroupRenter;
use App\Model\Room\RoomElectric;
use App\Model\Group\Group;
use App\Model\Renter\Renter;
use App\Model\Group\GroupInvoice as Invoice;
use App\Model\Group\GroupRoom;
use App\Model\Room\RoomAvaiable as Avaiable;
use App\Model\Room\RoomWater;
use App\Model\Setting\Unity;

use Carbon\Carbon;
use Dingo\Api\Auth\Auth as AuthAuth;

class RoomController extends ApiController
{
    use Helpers;
   
    function listing(Request $req){
       
        $data           = Room::select('*')
        ->with([

            'rGroups:id,room_id,group_id',
            'rGroups.group:id',
            'rGroups.group.gRenters:id,group_id,renter_id,is_primary_contact',
            'rGroups.group.gRenters.renters:id,name,phone',
            'type:id,name',
            'zone:id,name'
        ]);
       
         // ==============================>> Code
         if( $req->code && $req->code !="" ){
            $data = $data->where('code', $req->code);
        }

         // ==============================>> Type
         if($req->type != ''){
            $data = $data->where('type_id',$req->type);
        }

        // ==============================>> Zone
        if($req->zone && $req->zone != ''){
            $data = $data->where('zone_id',$req->zone);
        }

        // ==============================>> price
        if($req->price && $req->price != ''){
            $data = $data->where('price',$req->price);
        }
        
        $data= $data->orderBy('id', 'desc')->paginate( $req->limit ? $req->limit :10);         
        return response()->json($data, 200); 
    }

    function view($id=0){
        $room            =Room::select('*')
        ->with([
            'type:id,name',
            'zone:id,name',
            'groups.gRenters.renters',
        ])
        ->find($id);

        if($room){
            return response()->json($room, 200);
        }else{
            return response()->json([
                'status'    =>'fail',
                'message'   =>'Record not found.'
            ], 404);
        }
    }
    function grouprenter($id = 0, Request $req){
        
        $data                       =GroupRoom::select('id','room_id','group_id','deposit','type_id','booked_date','check_in_date','check_out_date')
        
        ->with([
            'type:id,name',
            'group:id',
            'group.gRenters:id,group_id,renter_id',
            'group.gRenters.renters:id,name,phone',
        ])
        ->where('room_id', $id);
    
        if($data){
        $data= $data->orderBy('id', 'desc')->paginate( $req->limit ? $req->limit : 10);         
         return response()->json($data, 200); 
        
        }else{
            return response()->json([
                'status'    =>'fail',
                'message'   =>'Record not found.'
            ], 404);
        }
    }
    function electric($id = 0, Request $req){

        $data                       = RoomElectric::select('*')
        ->where('room_id', $id);
        if($data){
            $data= $data->orderBy('id', 'desc')->paginate( $req->limit ? $req->limit : 10);
            return response()->json($data, 200);          
        }else{
            return response()->json([
                'status'    =>'fail',
                'message'   =>'Record not found.'
            ], 404);
        }
    }

    function AddElectric($id = 0, Request $req){
        $user = JWTAuth::parseToken()->authenticate();
        $this->validate($req, [
            
            'number'                  => 'required|max:50',
        ],[
            'number.required'         => 'Please add new number',

        ]);

        $unity    = Unity::where('name','electric')->first();
      
        $data                       =   New RoomElectric; 
        $data->number               = $req->input('number');
        $data->room_id              = $id;
        $data->price                = $unity->sell_out_price;
        $data->creator_id           = $user->id;
        $data->save();
        return response()->json([
            'status'    =>'success',
            'message'   =>'Add new number successful',
            'data' => $data,
        ], 200);
        
    }

    function water($id = 0, Request $req){

        $data                       = RoomWater::select('id','number','room_id','price','is_invoice_made','made_invoice_at','maker_id','creator_id','created_at','updated_at')
        ->with([
            'enterer:id,name'
        ])
        ->where('room_id',$id);
        if($data){
            $data= $data->orderBy('id', 'desc')->paginate( $req->limit ? $req->limit : 10);
            return response()->json($data, 200);
        }else{
            return response()->json([
                'status'    =>'fail',
                'message'   =>'Record not found.'
            ], 404);
        }
    }

    function AddWater($id = 0, Request $req){
        $user = JWTAuth::parseToken()->authenticate();
        $this->validate($req, [
            
            'number'                  => 'required|max:50',
        ],[
            'number.required'         => 'Please add new number',

        ]);

        $unity    = Unity::where('name','water')->first();
      
        $data                       =   New RoomWater; 
        $data->number               = $req->input('number');
        $data->room_id              = $id;
        $data->price                = $unity->sell_out_price;
        $data->creator_id          = $user->id;
        $data->save();
        return response()->json([
            'status'    =>'success',
            'message'   =>'Add new number successful',
            'data' => $data,
        ], 200);
        
    }

    function getfree(Request $req){
        $data       = RoomAvaiable::select('id','room_id','created_at','from','is_booked')
        ->with([

            'room:id,type_id,code,deposit,price,zone_id',
            'room.type:id,name',
            'room.zone:id,name'
          
        ])
        
        ->where('status', 1);
        
        $data= $data->orderBy('id', 'desc')->paginate( $req->limit ? $req->limit : 10);         
        return response()->json($data, 200); 
    }

    function create(Request $request){

        $next_id = 1 ;
            $number_of_rooms = count(Zone::find($request->input('zone_id'))->rooms);
            if($number_of_rooms != 0){
                $next_id = $number_of_rooms+1;
            }

            $number_of_creation = $request->input('n_of_creation');
            $now                = date('Y-m-d H:i:s');
            $zone = Zone::find($request->input('zone_id'))->name;
            if($number_of_creation){
                // $i = 0;
                for($i=0 ; $i<$number_of_creation ; $i++){

                    // echo "Hello\n";
                    
                    $code = $zone."-".($next_id+$i);
                    $room                   = new Room;
                    $room->code             = $code;
                    $room->type_id          = $request->input('type_id');
                    $room->zone_id          = $request->input('zone_id');
                    $room->deposit          = $request->input('deposit');
                    $room->price            = $request->input('price');
                    $room->s1               = $request->input('s1');
                    $room->s2               = $request->input('s2');
                    $room->s                = $request->input('s');
                    $room->save();

                    if($room){
                        
                        $avaiable           = new RoomAvaiable;
                        $avaiable->room_id  =$room->id;
                        $avaiable->from     =$now;
                        $avaiable->status   = 1;
                        $avaiable->save();

                    }
                      
                }
                return response()->json([
                    'status' => 'success',
                    'message' => 'Data has been created',
                ]);
            }
            
            
           
        
    }

    function update(Request $req,  $id = 0 ){
        $this->validate($req, [
            
            'type_id'               => 'required',
            'zone_id'               => 'required',
            'deposit'               => 'required', 
            'price'                 => 'required',
            
        ],[
            'type_id.required'                  => 'Please input type',
            'zone_id.required'                  => 'Please input Zone',
            'deposit.required'                  => 'Please input deposit',
            'price.required'                    => 'Please input price',
            
        ]);

        $data                       = Room::find($id);
    
        if($data){

        $data->type_id              = $req->input('type_id');
        $data->zone_id              = $req->input('zone_id');
        $data->deposit              = $req->input('deposit');
        $data->price                = $req->input('price');
        $data->s1                   = $req->input('s1');
        $data->s2                   = $req->input('s2');
        $data->s                    = $req->input('s');

        $data->save();

            return response()->json([
                'status'            => 'success',
                'message'           => 'បានបង្កើតដោយជោគជ័យ',
                'Room'              => $data,

            ], 200);
        }else{
            return response()->json([
                'status' => 'No Expense record',
                'message' => 'Invalid room'
            ]);
        }

    }

    function delete($id = 0){
        $menu = Room::find($id);
        if($menu){

        $menu->delete();

        return response()->json([
            'status' => 'success',
            'message' => 'Menu has been deleted',
        ], 200);
        }else{
            return response()->json([
                'status' => 'No Expense record',
                'message' => 'Invalid room'
            ]);
        }
       
    }

    function listzone(){
       
        $zone           = Zone::select('id','name')
        ->withcount([
            'rooms',
            'rooms as paybled_room'=>function ($query) {
               
                $query->whereHas('avaiables', function ($q) {
                     $q->where('status', 1);
                });
            }
            ])
        ->with([
            'rooms',
            'rooms.avaiables'
        ])
        ->get();
        return $zone; 
    }
    function listtype(){
        $type           = RoomType::select('id','name')->get();
        return $type;
    }
    function book(Request $request){

        $room_id        = $request->input('room_id');
        $avaiable_id    = $request->input('avaiable_id');
        $name           = $request->input('name');
        $phone          = $request->input('phone');
        $checkin        = $request->input('checkin');
        $user_id        = Auth::id();
        $now            = date('Y-m-d H:i:s');

        $room = Room::find($room_id);
       

        $renter_id = 0;
        $group_id = 0;

        //Check if phone number has already existed
        $renter = Renter::select('*')->where('phone', $phone)->first();
        if($renter){ //Renter has already existed
            //Find that renter
            $renter_id = $renter->id;
            return response()->json([
                'status' => 'Phone already in use',
                'message' => 'លេខមួយនេះត្រូវបានប្រើរួចហើយ',
                'data'=>$renter
            ],200);
        }else{ // Add new renter
            $dataRenter =   array(
                                'name'   => $name,
                                'phone'  => $phone,
                                'creator_id' => $user_id,
                                'updater_id' => $user_id,
                                'created_at' =>  $now, 
                                'updated_at' =>  $now
                            );
            $renter_id = Renter::insertGetId($dataRenter);
        }

        //create new group 
        $dataGroup = array(
                            'creator_id' => $user_id,
                            'updater_id' => $user_id,
                            'created_at' =>  $now, 
                            'updated_at' =>  $now
                            ); 
        $group_id = Group::insertGetId($dataGroup);
        $group = Group::find($group_id);
        //add the existing renter as primary contact
        $dataGroupRenter =  array(
                                'group_id'   => $group_id,
                                'renter_id'  => $renter_id,
                                'check_in_date'  => $checkin,
                                'is_primary_contact' => 1,
                                'creator_id' => $user_id,
                                'updater_id' => $user_id,
                                'created_at' =>  $now, 
                                'updated_at' =>  $now
                            );
        $group->gRenters()->insert($dataGroupRenter);
        //Create group room and mark as booking
        $dataGroupRoom = array(
                                'group_id'   => $group_id,
                                'room_id'  => $room_id,
                                'type_id'  => 1, // 1 = booked; 
                                'booked_date' => date('Y-m-d'),
                                'check_in_date' => $checkin,
                                'deposit' => $room->deposit,
                                'creator_id' => $user_id,
                                'updater_id' => $user_id,
                                'created_at' =>  $now, 
                                'updated_at' =>  $now
                            );
        $group->gRooms()->insert($dataGroupRoom);
        //Create Invoice to the group
        $dataGroupInvoice = array(
                                // 'rate_id'   =>   rate(),
                                'group_id'   => $group_id,
                                'is_paid'  => 0,
                                'due_date' => date('Y-m-d'),
                                'creator_id' => $user_id,
                                'updater_id' => $user_id,
                                'created_at' =>  $now, 
                                'updated_at' =>  $now
                            );
        $invoice_id = Invoice::insertGetId($dataGroupInvoice);
        $invoice = Invoice::find($invoice_id);
        //Add Invoice temis (deposite price only); 
        
        $dataGroupInvoiceDetail = array(
                                'invoice_id'   => $invoice_id,
                                'type_id' => 1, // 1 = book room or make deposit
                                'usd_amount' => $room->deposit,
                                'riel_amount' => 0,
                                'qty' => 1,
                                'description' => 'កក់បន្ទប់ #'.$room->code,

                                'usd_unit_price' => $room->deposit,
                                'riel_unit_price' => 0,
                                'print_description' => 'កក់បន្ទប់​ #'.$room->code,

                                'creator_id' => $user_id,
                                'updater_id' => $user_id,
                                'created_at' =>  $now, 
                                'updated_at' =>  $now
                            );
        $invoice->details()->insert($dataGroupInvoiceDetail);
        //Make the room unavialbe  or busy 
        Avaiable::where('id', $avaiable_id)->update(['status'=>0, 'updater_id'=>$user_id]);
        //Respond back
         Session::flash('message', 'Booking has been completed.' );
        return response()->json([
            'status' => 'success',
            'message' => 'Booking has been completed.'
        ]);
    }
    function checkout(Request $req, $id=0){
        $user = JWTAuth::parseToken()->authenticate();
        $checkout =$req->input('check_out_date');
        $note = $req->input('note');
        $now            = date('Y-m-d H:i:s');
        $data               = Room::find($id);
        if($data){
            
            $dataCheckOut = array(
                'check_out_date' =>$checkout,
                'type_id' =>   3,
                'note' =>$note,
                'updater_id' => $user->id,
                'updated_at' =>now(),
            );
            GroupRoom::where('room_id', $id)->update($dataCheckOut);
            $group=GroupRoom::where('room_id',$data->id)->first();
            Group::find($group->group_id)->gRenters()->where('check_out_date', Null)->update(['check_out_date'=>$checkout]);
            
             // Add expense for withdraw
            $dataExpense = array(
                'amount' =>   $data->deposit,
                'type_id' =>   2, 
                'referal_id' =>   $id,
                'description' =>   $note,
                'creator_id' => $user->id, 
                'updater_id' => $user->id,
                'updated_at' =>   now(),
                'created_at' =>  now(),
            );
            Expense::insertGetId($dataExpense);
            $avaiables = $data->avaiables()->where('status', 1)->get();
            if($avaiables){
                 $data       = array(
                        'room_id' =>  $data->id,
                        'from' =>  $checkout,
                        'status' =>  1,
                        'creator_id' =>  $user->id,
                        'updater_id' =>  $user->id,
                        'deleter_id' =>  1,
                        'created_at' =>  now(), 
                        'updated_at' =>  now(),
                    );
                Avaiable::insertGetId($data);
            }
    
    
            Session::flash('message', 'Check out is completed!');
            return response()->json([
                'status' => 'success',
                'message' => 'Check out is completed!'
            ]);
    }
}
  
    // public function bookedCancel(Request $request,  $id = 0 ){
        
    //     $group_room_id = $request->input('book_id');
    //     $room_id=isset($_GET['room_id'])?$_GET['room_id']:0;
      
    //     $user_id        = Auth::id();
    //     $now            = date('Y-m-d H:i:s');

        
    //     $dataCancel = array(
    //         'is_book' =>   0, // 0 = cancel
    //         'updater_id' => $user_id,
    //         'updated_at' =>   $now,
    //     );
    //     Avaiable::where('id', $group_room_id)->update($dataCancel);
    //     $room_id = Avaiable::find($group_room_id)->room_id;
    //      //Make room as avaiable
    //     $room =  Room::find($room_id);
    //     $avaiables = $room->avaiables()->where('status', 1)->get();
    //     if(count($avaiables) < 1){
    //          $data       = array(
    //                 'room_id' =>  $room_id,
    //                 'from' =>  Date('Y-m-d'),
    //                 'status' =>  1,
    //                 'creator_id' =>  $user_id,
    //                 'updater_id' =>  $user_id,
    //                 'deleter_id' =>  1,
    //                 'created_at' =>  $now, 
    //                 'updated_at' =>  $now
    //             );
    //         Avaiable::insertGetId($data);
    //     }

    //     return response()->json([
    //         'status' => 'success',
    //         'msg' => 'Booking has been cancel'
    //     ]);
    // }


}