<?php

namespace App\Api\V1\Controllers\CP;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\Validator;
use Illuminate\Validation\Rule;

use Dingo\Api\Routing\Helpers;
use JWTAuth;


use App\Api\V1\Controllers\ApiController;
  
use App\Model\Staff\Staff;


use Carbon\Carbon;

class StaffController extends ApiController
{
    use Helpers;
   
    function listing(Request $req){
       
        $data           = Staff::select('*')
        ->with([

            'position:id,name',
        ]);

         // ==============================>> Type
         if($req->position != ''){  
            $data = $data->where('position_id',$req->position);
        }

        // ==============================>> Keyword
        if($req->key && $req->key != ''){
            $data = $data->where('name', 'LIKE', '%'.$req->key.'%')->orWhere('code', 'like', $req->key)->orWhere('phone', 'like', $req->key); 
        }

        $data= $data->orderBy('id', 'desc')->paginate( $req->limit ? $req->limit : 10);         
        return response()->json($data, 200); 
    }

    function view($id=0){
        $Staff            =Staff::select('*')->find($id);

        if($Staff){
            return response()->json($Staff, 200);
        }else{
            return response()->json([
                'status'    =>'fail',
                'message'   =>'Record not found.'
            ], 404);
        }
    }

    function create(Request $req){

        $this->validate($req, [
            
            'name'                  => 'required|max:50',
            'position_id'           => 'required',
            'phone'                 =>  [
                                        'required', 
                                        'regex:/(^[0][0-9].{7}$)|(^[0][0-9].{8}$)/', 
                                        Rule::unique('staffs', 'phone')
                                    ], 
            'sex'                   => 'required|max:10',
            'salary'                => 'required|max:10',
            ],[
                'name.required'     => 'Please input staff name',
                'name.max'          => 'Name not more than 20 letters',
                'position_id.required'=> 'Please input position',
                'phone.required'    => 'Please input phone number',
                'phone.regex'       => 'It not alow for letter or'
            ]);

        $data                       =   New Staff; 
        $data->position_id          = $req->input('position_id');
        $data->code                 = $this->generateCode();
        $data->name                = $req->input('name');
        $data->phone                = $req->input('phone');
        $data->sex                  = $req->input('sex');
        $data->email                = $req->input('email');
        $data->address              = $req->input('address');
        $data->salary               = $req->input('salary');

        $data->save(); 
        
        return response()->json([
            'status' => 'success',
            'Staff' => $data,
            'message' => 'ទិន្នន័យដែលបានបង្កើតថ្មី'
        ], 200);
        
    }

    function update(Request $req,  $id = 0 ){
        $this->validate($req, [
            'name'              => 'required|max:50',
            'position_id'       => 'required',
            'phone'             =>  [
                                     'required', 
                                     'regex:/(^[0][0-9].{7}$)|(^[0][0-9].{8}$)/', 
                                     
                                    ], 
            'sex'               => 'required|max:10',
            'salary'            => 'required|max:10',   

            ],[

                'name.required'         => 'Please input type',
                'phone.required'        => 'Please input Phone number',
                'phone.regex'           => 'Please input correct number',
               

                'sex.required'          => 'Please input sex',
                'salary.required'       => 'Please input salary',
               
            ]);
        $data               = Staff::find($id);
    
        if($data){

            $data->name                 = $req->input('name');
            $data->position_id          = $req->input('position_id');
            $data->phone                = $req->input('phone');
            $data->sex                  = $req->input('sex');
            $data->email                = $req->input('email');
            $data->address              = $req->input('address');
            $data->salary               = $req->input('salary');

            $data->save();

            return response()->json([
                'status'            => 'success',
                'message'           => 'បានបង្កើតដោយជោគជ័យ',
                'staff'            => $data,

            ], 200);
        }else{
            return response()->json([
                'status' => 'No data record',
                'message' => 'Invalid staff'
            ]);
        }

    }

    function delete($id = 0){
        $menu = Staff::find($id);
        if($menu){

        $menu->delete();

        return response()->json([
            'status' => 'success',
            'message' => 'Data has been deleted',
        ], 200);
        }else{
            return response()->json([
                'status' => 'No data record',
                'message' => 'Invalid staff'
            ]);
        }
       
    }

   function generateCode(){
        
        $lastCode = Staff::select('id')->orderBy('id', 'DESC')->first(); 
        $code = 1;
        if($lastCode){
            $code = $lastCode->id+1; 
        }

        if( $code >= 1 && $code < 100 ){
            $code = 'SFF-'.$code; 
        }else  if( $code >= 100 && $code < 1000000 ){
             $code = 'SFF-'.$code; 
        }
        return $code; 
    }
    

}