<?php

namespace App\Api\V1\Controllers\CP;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\Validator;
use Illuminate\Validation\Rule;



use Dingo\Api\Routing\Helpers;
use JWTAuth;


use App\Api\V1\Controllers\ApiController;
use App\CamCyber\FileUpload;
use App\Model\User\User;
use Carbon\Carbon;

class UserController extends ApiController
{
    use Helpers;
   
    function listing(Request $req){
       
        $data           = User::select('*')
        ->with([

            'position:id,name',
            'zoneUsers.zone',
            'userPermisions.permision',
            'logs'
            

        ])
        ->get();
          
        return $data; 
    }

    function view($id=0){
        $Staff            =User::select('*')->find($id);

        if($Staff){
            return response()->json($Staff, 200);
        }else{
            return response()->json([
                'status'    =>'fail',
                'message'   =>'Record not found.'
            ], 404);
        }
    }

    function create(Request $req){

        $this->validate($req, [
            
            'name'                  => 'required|max:50',
            'position_id'           => 'required',
            'phone'                 =>  [
                                        'required', 
                                        'regex:/(^[0][0-9].{7}$)|(^[0][0-9].{8}$)/', 
                                        Rule::unique('users', 'phone')
                                    ], 
           
            'password'              => 'required|min:6|max:20',
            'confirm_password'      => 'required|same:password',
            'email'                 =>[
                                        'required',
                                        Rule::unique('users','email')
                                    ] 
            ],[

                'name.required'     => 'Please input user name',
                'name.max'          => 'Name not more than 20 letters',
                'position_id.required'=> 'Please input position',
                'phone.required'    => 'Please input phone number',
                'phone.regex'       => 'Please input correct phone number',
                'phone.unique'      => 'This number is already to use',
                'password.required' => 'Please input password',
                'password.min'      => 'Password cannot be less than 6 characters',
                'password.max'      => 'Password no longer than 20 characters',
                'confirm_password.same'     => 'Please input correct password',
                'confirm_password.required' => 'Please input confirm password',
                'email.required'    => 'Please input email',
                'email.unique'      => 'This email is already to use',
            ]);

        $data                       =   New User; 
        $data->position_id          = $req->input('position_id');
        $data->name                 = $req->input('name');
        $data->phone                = $req->input('phone');
        $data->email                = $req->input('email');
        $data->password             = $req->input('password');
        $data->status               = 0;
        $data->is_active            = 1;
        $data->visible              = 1;
       
        $date_today = Carbon::today()->format('d').'-'.Carbon::today()->format('M').'-'.Carbon::today()->format('Y');    
        if($req->avatar){
            $avatar     = FileUpload::uploadFileV2( $req->avatar, 'users/'.$date_today , '');
            if($avatar['url']){
                $data->avatar = $avatar['url'];
            }
        }
        $data->save(); 
    
        return response()->json([
            'status' => 'success',
            'users' => $data,
            'message' => 'ទិន្នន័យដែលបានបង្កើតថ្មី'
        ], 200);
        
    }

    function update(Request $req,  $id = 0 ){
        $this->validate($req, [
            'name'                  => 'required|max:50',
            'position_id'           => 'required',
            'phone'                 =>  [
                                        'required', 
                                        'regex:/(^[0][0-9].{7}$)|(^[0][0-9].{8}$)/', 
                                       
                                    ], 
           
            ],[

                'name.required'     => 'Please input user name',
                'name.max'          => 'Name not more than 20 letters',
                'position_id.required'=> 'Please input position',
                'phone.required'    => 'Please input phone number',
                'phone.regex'       => 'Please input correct phone number',

               
               
            ]);
        $data               = User::find($id);
    
        if($data){

            $data->position_id          = $req->input('position_id');
            $data->name                 = $req->input('name');
            $data->phone                = $req->input('phone');
            $data->email                = $req->input('email');
            $data->status               = $req->input('status');
            $data->is_active            = 1;
            $data->visible              = 1;
   
            $date_today = Carbon::today()->format('d').'-'.Carbon::today()->format('M').'-'.Carbon::today()->format('Y');    
            if($req->avatar){
                $avatar     = FileUpload::uploadFileV2( $req->avatar, 'users/'.$date_today , '');
                if($avatar['url']){
                    $data->avatar = $avatar['url'];
                }
            }
           
            $data->save();

            return response()->json([
                'status'            => 'success',
                'message'           => 'Data have been updated',
                'User'            => $data,

            ], 200);
        }else{
            return response()->json([
                'status' => 'No user record',
                'message' => 'Invalid user'
            ]);
        }

    }

    function delete($id = 0){
        $menu = User::find($id);
        if($menu){

        $menu->delete();

        return response()->json([
            'status' => 'success',
            'message' => 'Menu has been deleted',
        ], 200);
        }else{
            return response()->json([
                'status' => 'No User record',
                'message' => 'Invalid staff'
            ]);
        }
       
    }
    

}