<?php

namespace App\Api\V1\Controllers\CP;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\Validator;

use Dingo\Api\Routing\Helpers;
use JWTAuth;
use Auth;
use DB;
use Session;

use App\Api\V1\Controllers\ApiController;
use App\Model\Group\GroupInvoice as Payment; 
use App\Model\Group\GroupInvoiceRecord as GInvoiceRecord;
use App\Model\Zone\Zone;
use App\Model\Zone\ZoneRecord;
use App\Model\Setting\Unity;
use App\Model\Setting\Rate;
use App\Model\Room\Room;




class ZoneController extends ApiController
{
    use Helpers;
   
    function listing(){
       
           
        $zone           = Zone::select('id','name')
        ->withcount([
            'rooms',
            'rooms as paybled_room'=>function ($query) {
                $query->whereHas('avaiables', function ($q) {
                     $q->where('status', 1);
                });
            }
            ])
        ->with([
            'rooms:id,type_id,zone_id,code,deposit,price,s1,s2,s',
            'rooms.type:id,name',
            'records:created_at,zone_id,id,creator_id',
            'records.user:id,name',
        ])
        ->get();
        return $zone; 
    }

    function view($id=0){
       
           
        $zone           = Zone::select('id','name')
        ->withcount([
            'rooms',
            'rooms as paybled_room'=>function ($query) {
                $query->whereHas('avaiables', function ($q) {
                     $q->where('status', 1);
                });
            }
            ])
        ->with([
            'rooms:id,type_id,zone_id,code,deposit,price,s1,s2,s',
            'rooms.type:id,name',
        
        ])
        ->find($id);


        $invoice           = ZoneRecord::select('*')
        ->withCount([
            'gi_records'
        ])
        ->with([
            'user:id,name'
        ])
        ->where('zone_id', $zone->id)
        ->orderBy('id', 'desc')
        ->get();

        if($zone){
            return [
                'zone' => $zone,
                'record' =>  $invoice,
            ];
        }else{
            return response()->json([
                'status'    =>'fail',
                'message'   =>'Record not found.'
            ], 404);
        }
    }


    function create(Request $req){

        $this->validate($req, [ 
            'name'                  => 'required|max:10',
        ],[

            'name.required'                 => 'Please input name ',
           
        ]);
        $data                       = New Zone();
        $data->name                 = $req->input('name');
        $data->save(); 
        return response()->json([
            'status' => 'success',
            'Staff' => $data,
            'message' => 'ទិន្នន័យដែលបានបង្កើតថ្មី'
        ], 200);
        
    }

    function update(Request $req,  $id = 0 ){
        $this->validate($req, [
            'name'                      => 'required|max:10',
        ],[
            'name.required'                  => 'Please input name ',
        ]);
        $data                           = Zone::find($id);
        if($data){
            $data->name                 = $req->input('name');
            $data->save();
            return response()->json([
                'status'            => 'success',
                'message'           => 'បានកែប្រែដោយជោគជ័យ',
                'Income'            => $data,
            ], 200);
        }else{
            return response()->json([
                'status' => 'No Expense record',
                'message' => 'Invalid Expense'
            ]);
        }
    }
    function delete($id = 0){
        $menu = Zone::find($id);
        if($menu){
        $menu->delete();
        return response()->json([
            'status' => 'success',
            'message' => 'Data has been deleted',
        ], 200);
        }else{
            return response()->json([
                'status' => 'No data record',
                'message' => 'Invalid Unity'
            ]);
        }
       
    }
    function record($id=0){
        $data          =GroupInvoice::select('id','group_id','due_date','created_at','creator_id','updater_id','is_paid')
        ->with([
            'details:id,invoice_id,type_id,usd_amount,riel_amount,created_at,print_description,qty,description',
            'group:id',
            'group.rooms',
            'group.rooms.zone:id,name',
            'group.rooms.waters:room_id,id,number',
            'group.renters',
            'user:id,name',
            'receive:id,name'
        ])
       
        ->withCount([
            'details as total_in_usd' => function($query){
                $query->select(DB::raw("SUM(usd_amount) as total_in_usd")); 
            },
            'details as total_in_riel' => function($query){
                $query->select(DB::raw("SUM(riel_amount) as total_in_riel")); 
            }
        ])
        ->where('id',$id);
       

        if($data){
            $data= $data->orderBy('id', 'desc')->limit(10)->get();
            return response()->json($data, 200);
        }else{
            return response()->json([
                'status'    =>'fail',
                'message'   =>'Record not found.'
            ], 404);
        }
    }
    function generate(Request $request, $id = 0){
        //return $id;
        $zone_id = $id;
        $next_week = date("Y-m-d", strtotime("+1 week"));
        $user_id    = Auth::id();
        $now        = date('Y-m-d H:i:s');

        
        $unity_price = Unity::where('slug', 'service')->select('sell_out_price as price')->first()->price;
        $tv_price = Unity::where('slug', 'tv')->select('sell_out_price as price')->first()->price;

        $data_zone_record       = array(
                    
                    'zone_id'   =>  $zone_id,
                    'creator_id' =>  $user_id,
                    'updater_id' =>  $user_id,
                    'deleter_id' =>  1,
                    'created_at' =>  $now, 
                    'updated_at' =>  $now
                );
        $zone_record_id=ZoneRecord::insertGetId($data_zone_record);
        Session::flash('msg', 'New Payment record has been made');

        $rooms = Room::select('*')->where('zone_id', $zone_id)->whereHas('rGroups', function ($query) {
                    $query->where('type_id', 2);
                })->get();

        $user_id        = Auth::id();
        $now            = date('Y-m-d H:i:s');
        $next_week      = date("Y-m-d", strtotime("+1 week"));
        $rate_id        = 1;
        $riel           = Rate::find($rate_id)->riel;

        foreach($rooms as $room){
            
            //Finding water records
            $old_water_record = 0;
            $old_water_date = '';
            $new_water_record = 0;
            $new_water_date = '';
            $water_record     = 0;
            $water_unit_price = 0;
            
            $last_paid_record           = $room->waters()->select('number', 'created_at')->where('is_invoice_made', 1)->orderBy('id', 'DESC')->first();
            if($last_paid_record){
                $old_water_record = $last_paid_record->number;
                $old_water_date = date_format($last_paid_record->created_at, 'Y-m-d');
            }

            $un_invoice_water_records   = $room->waters()->select('number', 'price', 'created_at')->where('is_invoice_made', 0)->orderBy('id', 'ASC')->get();
            $num_of_un_invoice_water_records = count($un_invoice_water_records);
            if($num_of_un_invoice_water_records != 0){
                if($num_of_un_invoice_water_records == 1 ){
                    $new_water_record = $un_invoice_water_records[0]->number;
                    $water_unit_price = $un_invoice_water_records[0]->price;
                    $new_water_date = date_format($un_invoice_water_records[0]->created_at, 'Y-m-d');
                }else{
                    $new_water_record = $un_invoice_water_records[$num_of_un_invoice_water_records-1]->number;
                    $water_unit_price = $un_invoice_water_records[$num_of_un_invoice_water_records-1]->price;
                    $new_water_date = date_format($un_invoice_water_records[$num_of_un_invoice_water_records-1]->created_at, 'Y-m-d');
                }
                
                $water_record = $new_water_record-$old_water_record;
                $room->waters()->where('is_invoice_made', 0)->update(['is_invoice_made'=>1]);
            }
           

            //Finding electric records
            $old_electric_record = 0; 
            $old_electric_date = ''; 
            $new_electric_record = 0;
            $new_electric_date = ''; 
            $electric_record     = 0;
            $electric_unit_price = 0;
            
            $last_paid_record           = $room->electrics()->select('number', 'created_at')->where('is_invoice_made', 1)->orderBy('id', 'DESC')->first();
            if($last_paid_record){
                $old_electric_record = $last_paid_record->number;
                $old_electric_date = date_format($last_paid_record->created_at, 'Y-m-d');
            }

            $un_invoice_electric_records    = $room->electrics()->select('number', 'price', 'created_at')->where('is_invoice_made', 0)->orderBy('id', 'ASC')->get();
            $num_of_un_invoice_electric_records = count($un_invoice_electric_records);
            if($num_of_un_invoice_electric_records != 0){
                if($num_of_un_invoice_electric_records == 1 ){
                    $new_electric_record = $un_invoice_electric_records[0]->number;
                    $electric_unit_price = $un_invoice_electric_records[0]->price;
                    $new_electric_date = date_format($un_invoice_electric_records[0]->created_at, 'Y-m-d');
                }else{
                    $new_electric_record = $un_invoice_electric_records[$num_of_un_invoice_electric_records-1]->number;
                    $electric_unit_price = $un_invoice_electric_records[$num_of_un_invoice_electric_records-1]->price;
                    $new_electric_date = date_format($un_invoice_electric_records[$num_of_un_invoice_electric_records-1]->created_at, 'Y-m-d');
                }
                
                $electric_record = $new_electric_record-$old_electric_record;
                $room->electrics()->where('is_invoice_made', 0)->update(['is_invoice_made'=>1]);
            }
            //echo 'ID: '.$room->id.' Old:'.$new_electric_record.' New:'.$new_electric_record.'<br />' ;

            // Make Invoice
            $group = $room->rGroups()->select('*')->where('type_id', 2)->orderBy('id', 'DESC')->first();

            if($group){
                $group_id = $group->group_id;
                $dataGroupInvoice = array(
                                'rate_id'       => $rate_id,
                                'group_id'      => $group_id,
                                'is_paid'       => 0,
                                'due_date'      => $next_week,
                                'unity_description'=>'',
                                'creator_id'    => $user_id,
                                'updater_id'    => $user_id,
                                'created_at'    =>  $now, 
                                'updated_at'    =>  $now
                            );
                $invoice_id = Payment::insertGetId($dataGroupInvoice);
                $dataGiRecord = array(
                                'record_id'       => $zone_record_id,
                                'payment_id'      => $invoice_id,
                                'creator_id'    => $user_id,
                                'updater_id'    => $user_id,
                                'created_at'    =>  $now, 
                                'updated_at'    =>  $now
                            );
                GInvoiceRecord::insert($dataGiRecord);

                $invoice = Payment::find($invoice_id);
                $dataUnityJson = array();

                //Room Price
                $dataGroupInvoiceDetail = array(
                                                    'invoice_id'   => $invoice_id,
                                                    'type_id' => 2, 
                                                    'usd_amount' => $room->price,
                                                    'riel_amount' => 0,
                                                    'qty' => 1,
                                                    'description' => 'ថ្លៃបន្ទប់ #'.$room->code,

                                                    'usd_unit_price' => $room->price,
                                                    'riel_unit_price' => 0,
                                                    'print_description' => 'ថ្លៃបន្ទប់ #'.$room->code,

                                                    'creator_id' => $user_id,
                                                    'updater_id' => $user_id,
                                                    'created_at' =>  $now, 
                                                    'updated_at' =>  $now
                                                );
                $invoice->details()->insert($dataGroupInvoiceDetail);
                //Water Pirce
                if($water_record > 0){
                    
                    $amount = ($water_unit_price*$water_record);
                    $description = 'លេខចាស់: '.$old_water_record.' - លេខថ្មី: '.$new_water_record.'<br /> ចំនួន (m<sup>3</sup>):'.$water_record.' <br /> តំលៃរាយ (R):'.$room->water;
                    $dataUnityJson['water'] = array('name'=>'ទឹក', 'old_number'=>$old_water_record, 'new_number'=>$new_water_record, 'qty'=>$water_record, 'old_date'=>$old_water_date, 'new_date'=>$new_water_date);

                    $dataGroupInvoiceDetail = array(
                                                    'invoice_id'   => $invoice_id,
                                                    'type_id' => 3, 
                                                    'usd_amount' => 0,
                                                    'riel_amount' => $amount,
                                                    'qty' => $water_record,
                                                    'description' => $description,

                                                    'usd_unit_price' => 0,
                                                    'riel_unit_price' => $water_unit_price,
                                                    'print_description' => 'ទឹក',

                                                    'creator_id' => $user_id,
                                                    'updater_id' => $user_id,
                                                    'created_at' =>  $now, 
                                                    'updated_at' =>  $now
                                                );
                    $invoice->details()->insert($dataGroupInvoiceDetail);

                }
                //Electric Pirce
                if($electric_record > 0){
                    $amount = ($electric_unit_price*$electric_record);
                    $description = 'លេខចាស់: '.$old_electric_record.' - លេខថ្មី: '.$new_electric_record.'<br /> ចំនួន (kwh):'.$electric_record.' <br /> តំលៃរាយ (R):'.$room->electric;
                    $dataUnityJson['electric'] = array('name'=>'ភ្លើង', 'old_number'=>$old_electric_record, 'new_number'=>$new_electric_record, 'qty'=>$electric_record, 'old_date'=>$old_electric_date, 'new_date'=>$new_electric_date);

                    $dataGroupInvoiceDetail = array(
                                                    'invoice_id'   => $invoice_id,
                                                    'type_id' => 4, 
                                                    'usd_amount' => 0,
                                                    'riel_amount' => $amount,
                                                    'qty' => $electric_record,
                                                    'description' => $description,

                                                    'usd_unit_price' => 0,
                                                    'riel_unit_price' => $electric_unit_price,
                                                    'print_description' => 'ភ្លើង',

                                                    'updater_id' => $user_id,
                                                    'created_at' =>  $now, 
                                                    'updated_at' =>  $now
                                                );
                    $invoice->details()->insert($dataGroupInvoiceDetail);
                }
                //Unity Service
                if($group->is_unity_paid == 1){
                    if(is_numeric($group->unity_price)){
                        $unity_price = $group->unity_price;
                    }
                    $dataGroupInvoiceDetail = array(
                                                    'invoice_id'   => $invoice_id,
                                                    'type_id' => 5, 
                                                    'usd_amount' => 0,
                                                    'riel_amount' => $unity_price,
                                                    'qty' => 1,
                                                    'description' => 'សេវាកម្មបងថ្លៃភ្លើង',

                                                    'usd_unit_price' => 0,
                                                    'riel_unit_price' => $unity_price,
                                                    'print_description' => 'សេវាកម្មបងថ្លៃភ្លើង',

                                                    'updater_id' => $user_id,
                                                    'created_at' =>  $now, 
                                                    'updated_at' =>  $now
                                                );
                    $invoice->details()->insert($dataGroupInvoiceDetail);
                }

                //TV
                if($group->is_tv_paid == 1){
                    if(is_numeric($group->tv_price)){
                        $tv_price = $group->tv_price;
                    }
                    $dataGroupInvoiceDetail = array(
                                                    'invoice_id'   => $invoice_id,
                                                    'type_id' => 6, 
                                                    'usd_amount' => $tv_price,
                                                    'riel_amount' => 0,
                                                    'qty' => 1,
                                                    'description' => 'ទូរទស្សន៍​ខ្សែកាប',

                                                    'usd_unit_price' => $tv_price,
                                                    'riel_unit_price' => 0,
                                                    'print_description' => 'ទូរទស្សន៍​ខ្សែកាប',

                                                    'updater_id' => $user_id,
                                                    'created_at' =>  $now, 
                                                    'updated_at' =>  $now
                                                );
                    $invoice->details()->insert($dataGroupInvoiceDetail);
                }

                //Garbage
                //if($group->is_garbage_paid == 1){
                    // if(is_numeric($group->garbage_price)){
                    //     $garbage_price = $group->garbage_price;
                    // }
                    $dataGroupInvoiceDetail = array(
                                                    'invoice_id'   => $invoice_id,
                                                    'type_id' => 7, 
                                                    'usd_amount' => 1,
                                                    'riel_amount' => 0,
                                                    'qty' => 1,
                                                    'description' => 'សេវាប្រមូលសំរាម',

                                                    'usd_unit_price' => 1,
                                                    'riel_unit_price' => 0,
                                                    'print_description' => 'សេវាប្រមូលសំរាម',

                                                    'updater_id' => $user_id,
                                                    'created_at' =>  $now, 
                                                    'updated_at' =>  $now
                                                );
                    $invoice->details()->insert($dataGroupInvoiceDetail);
                //}

                if(count($dataUnityJson) > 0){
                   $json = json_encode($dataUnityJson);
                    Payment::where('id', $invoice_id)->update(['unity_description'=>$json]);
                }
                
            }
            
        }
    }

}