<?php

namespace App\Api\V1\Controllers\Client;

use Auth;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Validator;
use Illuminate\Validation\Rule;

use Dingo\Api\Routing\Helpers;
use JWTAuth;

use App\Api\V1\Controllers\ApiController;

use App\Model\Zone\Zone; 
use App\Model\Room\Room; 
use App\Model\Room\RoomElectric as RElectric;
use App\Model\Room\RoomWater as RWater; 

use App\Model\Group\GroupRoom; 
use App\Model\Setting\Unity; 

class ZoneController extends ApiController
{
    use Helpers;
    
    //================================>> List all Zones and Room
    function listing(){
       
        $data           = Zone::select('id','name')
        ->with([
            'rooms:id,type_id,zone_id,code,deposit,price'
        ])
        ->get();
        return $data; 
    }

    //================================>> View Single Room
    function viewRoom($room_id = 0){
       
        // Check Room
        $room = Room::select('id','type_id','zone_id','code','deposit','price','s1','s2','s')->find($room_id);
        if($room){

            $group = GroupRoom::select('id','group_id','room_id','type_id', 'water_price', 'is_speical_water_paid', 'electric_price', 'is_special_electric_paid','booked_date','check_in_date')
            ->where('room_id', $room_id)
            ->with([ 'group:id'])
            ->first();

            if($group){
                if($group->is_speical_water_paid != 1){
                    $group->water_price = Unity::select('sell_out_price as price')->where('slug', 'water')->first()->price; 
                }

                if($group->is_special_electric_paid != 1){
                    $group->electric_price = Unity::select('sell_out_price as price')->where('slug', 'electric')->first()->price; 
                }

                $group->renters = $group->group->gRenters; 
                unset($group->group); 
            }

            $waters = RWater::select('id','room_id','number','price','is_invoice_made','made_invoice_at','created_at')
            ->where('room_id', $room_id)
            ->orderBy('id', 'DESC')
            ->limit(5)
            ->get();

            $electrics = RElectric::select('id','room_id','number','price','is_invoice_made','made_invoice_at','created_at')
            ->where('room_id', $room_id)
            ->orderBy('id', 'DESC')
            ->limit(5)
            ->get();

            $lastElectricRecord = 0;
            if (count($electrics) > 0) {
                $lastElectricRecord = $electrics[0]->number;
            } 

            $lastWaterRecord = 0;
            if (count($waters) > 0) {
                $lastWaterRecord = $waters[0]->number;
            }

            return [
                'room'                   => $room,
                'group'                  => $group,
                'waters'                 => $waters,
                'electrics'              => $electrics,
                'last_electric_record'   => $lastElectricRecord,
                'last_water_record'      => $lastWaterRecord,
            ]; 

        }else{
            return response()->json([
                'message' => 'Invalid Room',
            ], 400);
        }
       
    }
    
    function resetElectric(Request $req){
        $user_id    = Auth::id();
        $this->validate($req, [

                'room_id'            => 'required',
                    
        ]);
        //Check if Valid Room
        $room = Room::select('id','type_id','zone_id','code','deposit','price','s1','s2','s')->find($req->room_id);
        if($room){

            $electricity                       =   New RElectric; 
            $electricity->room_id              =   $req->input('room_id'); 
            $electricity->number               =   0; 
            $electricity->price                =   0;
            $electricity->is_invoice_made      =   1;
            $electricity->creator_id           =   $user_id;
            $electricity->save();
                            

            return response()->json([
                'status'         => 'success',
                'electricity'    => $electricity,
                'message'        => 'ទិន្ន័យអគ្គិសនីត្រូវបានចាប់ផ្ដើមពី 0'
            ], 200);
            
           

        }else{
            return response()->json([
                'message' => 'Invalid Room',
            ], 400);
        }

    }

    //================================>> Wate Record
    function addUpdateWater(Request $req){
        $user_id    = Auth::id();
        $this->validate($req, [
                'room_id'            => 'required',     
        ]);

        //Check if Valid Room
        $room = Room::select('id','type_id','zone_id','code','deposit','price','s1','s2','s')->find($req->room_id);
        if($room){
            $roomData = $this->viewRoom($req->room_id); 

                // Check Eletric
                $lastWaterRecord = isset($roomData['last_water_record']) ? $roomData['last_water_record'] : 0; 
                if($lastWaterRecord < $req->new_number){
                   
                    $water                       =   New RWater; 
                    $message = 'ទិន្ន័យទឹកត្រូវបានបង្កើតថ្មី';
                    //Check if user want to update or add new
                    if($req->id){ //update
                        $checkRecord = RWater::find($req->id); 
                        if($checkRecord){
                            $water = $checkRecord; 
                            $message = 'ទិន្ន័យទឹកត្រូវបានកែប្រែរ';
                        }else{
                            return response()->json([
                                'status'    => 'fail',
                                'message'   => 'រកមិនឃើញទិន្ន័យ',
                            ], 200);
                        }
                    }
                   
                    $water->room_id              =   $req->input('room_id'); 
                    $water->number               =   $req->input('new_number'); 
                    $water->price                =   $roomData['group']->water_price;
                    $water->is_invoice_made      =   0;
                    $water->creator_id           =  $user_id;
                    $water->save();
  
                    return response()->json([
                        'status'         => 'success',
                        'water'          => $water,
                        'creator_id' =>  $user_id,
                        'message'        => $message
                    ], 200);

                }else{
                    return response()->json([
                        'message' => 'កំណត់ត្រាទឹកថ្មីត្រូវតែធំជាងមុន',
                        'last_record' => $roomData['last_water_record']
                    ], 400);
                }
            

            return $roomData; 
            // Cehck lastly record

           
        }else{
            return response()->json([
                'message' => 'Invalid Room',
            ], 400);
        }
        
    }

    //================================>> Wate Record
    function addUpdateElectric(Request $req){
        $user_id    = Auth::id();
        $this->validate($req, [
                'room_id'            => 'required',     
        ]);

        //Check if Valid Room
        $room = Room::select('id','type_id','zone_id','code','deposit','price','s1','s2','s')->find($req->room_id);
        if($room){
            $roomData = $this->viewRoom($req->room_id); 

                // Check Eletric
                $lastElectricRecord = isset($roomData['last_water_record']) ? $roomData['last_electric_record'] : 0; 
                if($lastElectricRecord < $req->new_number){
                   
                    $electric                       =   New RElectric; 
                    $message = 'ទិន្ន័យទឹកត្រូវបានបង្កើតថ្មី';
                    //Check if user want to update or add new
                    if($req->id){ //update
                        $checkRecord = RElectric::find($req->id); 
                        if($checkRecord){
                            $electric = $checkRecord; 
                            $message = 'ទិន្ន័យទឹកត្រូវបានកែប្រែរ';
                        }else{
                            return response()->json([
                                'status'    => 'fail',
                                'message'   => 'រកមិនឃើញទិន្ន័យ',
                            ], 200);
                        }
                    }
                   
                    $electric->room_id              =   $req->input('room_id'); 
                    $electric->number               =   $req->input('new_number'); 
                    $electric->price                =   $roomData['group']->electric_price;
                    $electric->is_invoice_made      =   0;
                    $electric->creator_id           =   $user_id;
                    $electric->save();
  
                    return response()->json([
                        'status'         => 'success',
                        'electric'       => $electric,
                        'creator_id'     => $user_id,
                        'message'        => $message
                    ], 200);

                }else{
                    return response()->json([
                        'message' => 'កំណត់ត្រាភ្លើងថ្មីត្រូវតែធំជាងមុន',
                        'last_record' => $roomData['last_electric_record']
                    ], 400);
                }
            

            return $roomData; 
            // Cehck lastly record

           
        }else{
            return response()->json([
                'message' => 'Invalid Room',
            ], 400);
        }
        
    }
    
    function resetWater(Request $req){
        $user_id    = Auth::id();
        $this->validate($req, [

                'room_id'            => 'required',
                    
        ]);
        //Check if Valid Room
        $room = Room::select('id','type_id','zone_id','code','deposit','price','s1','s2','s')->find($req->room_id);
        if($room){

            $water                       =   New RWater; 
            $water->room_id              =   $req->input('room_id'); 
            $water->number               =   0; 
            $water->price                =   0;
            $water->is_invoice_made      =   1;
            $water->creator_id           =   $user_id;
            $water->save();
                            

            return response()->json([
                'status'         => 'success',
                'water'    => $water,
                'message'        => 'ទិន្ន័យទឹកត្រូវបានចាប់ផ្ដើមពី 0'
            ], 200);
            
           

        }else{
            return response()->json([
                'message' => 'Invalid Room',
            ], 400);
        } 
        
    }

    function deleteWatter($id = 0) {
        $water = RWater::find($id);
        // check if record exist
        if($water) {
            // check if record can be delete
            if($water->is_invoice_made == 0) {
                $water->delete();
                return response()->json([
                    'status' => 'success',
                    'message' => 'ទិន្ន័យត្រូវបានលុប',
                ], 200);
            }else{
                return response()->json([
                    'status'  => 'fail',
                    'message' => 'ទិន្ន័យនេះមិនអាចលុបបានទេ',
                    'water' => $water,
                ], 400);
            }
        }else{
            return response()->json([
                'status'  => 'fail',
                'message' => 'ទិន្ន័យមិនត្រឹមត្រូវ',
            ], 400);
        }
       
    }

   function deleteElectricity($id = 0) {
    $electric = RElectric::find($id);
    // check if record exist
    if($electric) {
        // check if record can be delete
        if($electric->is_invoice_made == 0) {
            $electric->delete();
            return response()->json([
                'status' => 'success',
                'message' => 'ទិន្ន័យត្រូវបានលុប',
            ], 200);
        }else{
            return response()->json([
                'status'  => 'fail',
                'message' => 'ទិន្ន័យនេះមិនអាចលុបបានទេ',
                'electric' => $electric,
            ], 400);
        }
    }else{
        return response()->json([
            'status'  => 'fail',
            'message' => 'ទិន្ន័យមិនត្រឹមត្រូវ',
        ], 400);
    }
   }
    
  
}
